/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#include "gepro.h"
#include "files.h"
#include "gui.h"
#include  "lang.h"

#define SAVE_HEXRECSIZE 32
#define DEBUG printf


void load_hex()
{
    FILE *f;
//    char d,state,l,r,i;
//    int addr;
    
    if(!(f=fopen(GET_FNAME,"r"))){
	dialog_box("Error","Nie powiodlo sie otwarcie pliku",BUTTON_OK,NULL);
	printf("[ER]Open file %s error\n",GET_FNAME);
	return;
    }
/* tutaj dorobic wczytywaie hexa */
    fclose(f);
}


void load_bin()
{
    int fsize;
    FILE *f;

    if(!GET_DEV_SIZE){
	dialog_box("Error","Nie wybrano ukladu, wpierw wybierz uklad",BUTTON_OK,NULL);
	DEBUG("[!!]Device size is 0!\n");
	return;
    }

    if(!(f=fopen(GET_FNAME,"r"))){
	dialog_box("Error","Nie powiodlo sie otwarcie pliku",BUTTON_OK,NULL);
	DEBUG("[!!]Open file %s error\n",GET_FNAME);
	return;
    }

    fseek(f,0L,SEEK_END);
    fsize = ftell(f);
    fseek(f,0L,SEEK_SET);    

    if(fsize > GET_DEV_SIZE){
	dialog_box("Error","Rozmiar pliku wiekszy od rozmiaru bufora, dla wybranego ukladu",BUTTON_OK,NULL);
	DEBUG("[!!]file size is greater then buffer size: %i > %i\n",fsize,GET_DEV_SIZE);
	return;
    }


    if(fread(GET_BUFFER,1,fsize,f) != fsize){
	dialog_box("Error","Blad odczytu pliku",BUTTON_OK,NULL);
	DEBUG("[!!]File %s read error\n",GET_FNAME);
    }
    fclose(f);
    stat_rfsh();
}

void save_bin()
{
    FILE *f;
    
    if(!(f=fopen(GET_FNAME,"w"))){
	dialog_box("Error","Nie powiodlo sie otwarcie pliku do zapisu",BUTTON_OK,NULL);
	printf("[ER]Open file %s to write error\n",GET_FNAME);
	return;
    }

    if(fwrite(GET_BUFFER,1,GET_DEV_SIZE,f) != GET_DEV_SIZE){
	dialog_box("Error","Blad zapisu pliku",BUTTON_OK,NULL);
	DEBUG("[!!]File %s write error\n",GET_FNAME);
    }

    fclose(f);
}

void save_hex()
{
    FILE *f;
    int i,j,addr,k;
    long checksum;
    
    if(!(f=fopen(GET_FNAME,"w"))){
	dialog_box("Error","Nie powiodlo sie otwarcie pliku",BUTTON_OK,NULL);
	printf("[ER]Open file %s error\n",GET_FNAME);
	return;
    }
    addr=0;
    k=GET_DEV_SIZE;
    for(j = 0; j < GET_DEV_SIZE; ){    
	i = k > SAVE_HEXRECSIZE ? SAVE_HEXRECSIZE : k;
	k -= SAVE_HEXRECSIZE;
	fprintf(f,":%02X%02X%02X00",i & 0xff, (addr >> 8) & 0xff, addr & 0xff);
	checksum = (addr >> 8) + (addr & 0xff) + i;
	addr += 16;
	for(; i > 0; i--){
	    fprintf(f,"%02X", BUFFER(j) & 0xff);
	    checksum += BUFFER(j++);
	}
	fprintf(f,"%02X\n",(unsigned char)(-checksum & 0xff));
    }
    fprintf(f,":00000001FF\n");
    fclose(f);
}


char hex_check(char *path)
{

    if(!path) return 0;
    if(!*path) return 0;
    if(strlen(path) < 5) return 0; 
    while(*path) path++;
    path--;
    if(*path != 'x' && *path != 'X') return 0;
    path--;
    if(*path != 'e' && *path != 'E') return 0;
    path--;
    if(*path != 'h' && *path != 'H') return 0;
    path--;
    if(*path != '.') return 0;

    return 1;
}

void load_fname()
{
    if(hex_check(GET_FNAME)) load_hex();
	else load_bin();

}

void save_fname()
{
    if(hex_check(GET_FNAME)) save_hex();
	else save_bin();
}
