/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "lang.h"
#include "gepro.h"
#include "files.h"
#include "buffer.h"
#include "willem.h"
#include "main.h"
#include "gui.h"
#include "chip.h"
#include "img_idx.h"
#include "parport.h"
#include "xpms.c"

typedef struct{
    int		idx;
    GtkWidget 	*wg;
    void	*next;
} fo_idx;

char		dialog_exit, dialog_code;
GtkWidget	*tst_chkbx[32];
void		*p_wgts[256];	/* przechowuje wskaniki do widgetw */
conf_state	program_state;
fo_idx		*fo_mg = NULL, *fo_mg_first = NULL;
char		fo_cpage = 3; /* bo 3 zakladki maja zostac */
fo_idx		*ao_mg = NULL, *ao_mg_first = NULL;
char		progress_bar_exit;

typedef struct
{
    int 	id;
    GdkPixmap	*image;
    void	*next;
} rimg;

rimg  *r_img_root = NULL;

void dynamic_widget_add(fo_idx **fo, fo_idx **ff ,int idx ,int type, char *name, int value, void *fnc, void *parm, char pos);
void dynamic_widget_purge(fo_idx **fo, fo_idx **ff);
void dialog_bt(GtkWidget *wg, char *name);
void add_action(int, int, char *, gchar*, void*, void*);
void device_menu_create(GtkWidget *);
/***************************************************************************************************/

void free_images()
{
    void *tmp;
    rimg *r_img;
    for(r_img = r_img_root; r_img;){
	 tmp = ((rimg *)r_img)->next;
	 free(r_img);
	 r_img = tmp; 
    }
}

int register_image(char **img)
{
    GdkBitmap *mask;
    rimg *r_img;
    rimg *new_image;
    int  idx;

    new_image = (rimg *)malloc(sizeof(rimg));
    new_image->next = NULL;
    new_image->image = gdk_pixmap_create_from_xpm_d(GET_WG(MAIN_WINDOW)->window, &mask, NULL, (gchar **)img);
    
    if(!r_img_root)
    {
	new_image->id = 1;
	r_img_root = new_image;
	atexit(free_images);
	return 1;
    }

    for(idx = 1, r_img = r_img_root; ((rimg *)r_img)->next; idx++ ) r_img = ((rimg *)r_img)->next;
    idx++;
    new_image->id = idx;
    ((rimg *)r_img)->next = new_image;
    return idx;
}

GdkPixmap *get_image(int id)
{
    rimg *r_img;
    for(r_img = r_img_root; r_img; r_img = ((rimg *)r_img)->next )
	if( ((rimg *)r_img)->id == id) return ((rimg *)r_img)->image;
    return NULL;
}

/***************************************************************************************************/
void stat_rfsh(void)
{
    char tmp_str[40];

    gtk_entry_set_text(GTK_ENTRY(GET_WG(EN_DEV_NAME)), get_chip()->chip_name);
    sprintf(tmp_str,"0x%x",get_chip()->dev_size); gtk_entry_set_text(GTK_ENTRY(GET_WG(EN_SIZE_HEX)),tmp_str);  
    sprintf(tmp_str,"0x%x",get_chip()->checksum); gtk_entry_set_text(GTK_ENTRY(GET_WG(EN_CHECKSUM)),tmp_str);  
    gtk_label_set(GTK_LABEL(GET_WG(LB_STATUS_VAL)),GET_STATUS);
    SET_C_IMAGE((GET_PCB_WILLEM) ? GET_C_WILLEM_IMAGE : GET_C_PCB3_IMAGE);
    draw_dip_switch();
    draw_pict();
    viewer_rfsh();
}

void exit_program()
{ 
    if(dialog_box(QUIT_TITLE,QUIT_MSG,BUTTON_NO,BUTTON_OK)==2) kill_me();
}

void sel_file_load(GtkFileSelection *wg, void *fs)
{
    char *tmp;
    
    tmp = (char*)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
    if(GET_FNAME) free(GET_FNAME);
    if(!(SET_FNAME ( (char*)malloc(sizeof(char)*(strlen(tmp) + 1))))){
	perror(ALLOCATION_ERROR);
	exit(1);
    }
    strcpy(GET_FNAME,tmp);
    load_fname();
}

void sel_file_save(GtkFileSelection *wg, void *fs)
{
    char *tmp;
    
    tmp = (char*)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
    if(GET_FNAME) free(GET_FNAME);
    if(!(SET_FNAME ( (char*)malloc(sizeof(char)*(strlen(tmp) + 1))))){
	perror(ALLOCATION_ERROR);
	exit(1);
    }
    strcpy(GET_FNAME,tmp);
    save_fname();
}

void load_file()
{ 
    GtkWidget *wg;    
    
    wg = gtk_file_selection_new(FSEL_LOAD);
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(wg));
    gtk_file_selection_complete(GTK_FILE_SELECTION(wg),"*.bin *.hex");
    if(GET_FNAME)
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(wg), GET_FNAME);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(wg)->ok_button),"clicked",
	GTK_SIGNAL_FUNC(sel_file_load),wg);
    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(wg)->ok_button),"clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy),wg);    
    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(wg)->cancel_button),"clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy),wg);    
    gtk_widget_show(wg);    
}

void save_file(){ 
    GtkWidget *wg;    

    wg = gtk_file_selection_new(FSEL_SAVE);
    if(GET_FNAME)
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(wg), GET_FNAME);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(wg)->ok_button),"clicked",
	GTK_SIGNAL_FUNC(sel_file_save),wg);
    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(wg)->ok_button),"clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy),wg);    
    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(wg)->cancel_button),"clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy),wg);    
    gtk_widget_show(wg);        
}

void device_sel(GtkWidget *wg, char *name) 
{ 
    chip *tmp = get_chip();
    
    if(!find_chip(name)){
	error_box(NO_CHIP_PLUGIN);
    } else { 
	if(tmp){
	    fast_option_purge();
	    adv_option_purge();
	    if(tmp->read_chip)
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_READ_CHIP)));
	    if(tmp->verify_chip)	    
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_VERIFY_CHIP)));
	    if(tmp->read_sig_chip)	
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_ID_CHIP)));
	    if(tmp->test_chip)	
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_BLANK_CHIP)));
	    if(tmp->write_chip)
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_PROGRAM_CHIP)));
	    if(tmp->erase_chip)	
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_ERASE_CHIP)));
	    if(tmp->lock_chip)	
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_LB_CHIP)));
	    if(tmp->unlock_chip)	
		gtk_widget_destroy(GTK_WIDGET(GET_WG(TB_RM_ULB_CHIP)));
	    free(tmp->buffer);
	}
	tmp = get_chip();
	if(!(tmp->buffer = (char *)malloc(tmp->dev_size))){
	    perror(MALLOC_ERROR);
	    exit(-1);
	}
	SET_BUFFER(tmp->buffer);
	SET_DEV_NAME(tmp->chip_name);
	SET_DEV_SIZE(tmp->dev_size);
	memset(GET_BUFFER,0,GET_DEV_SIZE);
	SET_ADAPTER_IMG(tmp->img_will_idx,tmp->img_pcb3_idx);

	GET_ADJ(WG_VIEWER_ADJ)->lower = 0;
	GET_ADJ(WG_VIEWER_ADJ)->upper = tmp->dev_size;
	GET_ADJ(WG_VIEWER_ADJ)->page_size = 25*16;
	GET_ADJ(WG_VIEWER_ADJ)->value = 0;
	gtk_adjustment_value_changed(GET_ADJ(WG_VIEWER_ADJ));

	if(!tmp->img_will_idx)
	    SET_C_WILLEM_IMAGE(tmp->image_willem);
	if(!tmp->img_pcb3_idx)
	    SET_C_PCB3_IMAGE(tmp->image_pcb);
	if(tmp->read_chip)
	    add_action(TB_P_READ_CHIP, TB_RM_READ_CHIP, GTK_STOCK_CONVERT, READ_CHIP_TIP, TB_READ_CHIP );
	if(tmp->verify_chip)	    
	    add_action(TB_P_VERIFY_CHIP, TB_RM_VERIFY_CHIP, GTK_STOCK_COPY, VERIFY_CHIP_TIP, TB_VERIFY_CHIP );
	if(tmp->read_sig_chip)	
	    add_action(TB_P_ID_CHIP, TB_RM_ID_CHIP, GTK_STOCK_COLOR_PICKER, ID_CHIP_TIP, TB_ID_CHIP );
	if(tmp->test_chip)	
	    add_action(TB_P_BLANK_CHIP, TB_RM_BLANK_CHIP, GTK_STOCK_DIALOG_QUESTION, BLANK_CHIP_TIP, TB_BLANK_CHIP );
	if(tmp->write_chip)	
	    add_action(TB_P_PROGRAM_CHIP, TB_RM_PROGRAM_CHIP, GTK_STOCK_EDIT, PROGRAM_CHIP_TIP, TB_PROGRAM_CHIP );
	if(tmp->erase_chip)	
	    add_action(TB_P_ERASE_CHIP, TB_RM_ERASE_CHIP, GTK_STOCK_DELETE, ERASE_CHIP_TIP, TB_ERASE_CHIP );
	if(tmp->lock_chip)	
	    add_action(TB_P_LB_CHIP, TB_RM_LB_CHIP, GTK_STOCK_DIALOG_AUTHENTICATION, LB_CHIP_TIP, TB_LB_CHIP );
	if(tmp->unlock_chip)	
	    add_action(TB_P_ULB_CHIP, TB_RM_ULB_CHIP, GTK_STOCK_CUT, ULB_CHIP_TIP, TB_ULB_CHIP );
	SET_DIP_SWITCH(tmp->dip_switch);

	if(tmp->autostart)
	    tmp->autostart();
	else
	  fast_option_add(FO_LABEL,NO_FO_MSG, 0 , NULL, NULL,0);
	stat_rfsh();
    }
}

void rfsh_gtk(void)
{
    while(gtk_events_pending()) gtk_main_iteration();
}


void about(void)
{ 
    dialog_box(ABOUT_TITLE,ABOUT,BUTTON_OK,NULL); 
}

GtkWidget *xpm_2_widget(GtkWidget *window, gchar **xpm_data)
{
    GdkBitmap *mask; /*?*/
    GdkPixmap *map_data;
    GtkWidget *pix_widget;
    
    map_data  = gdk_pixmap_create_from_xpm_d(window->window,&mask,NULL,(gchar**)xpm_data);
    pix_widget = gtk_pixmap_new(map_data,mask);
    return pix_widget;
}

gint configure_pict(GtkWidget *wg,GdkEventConfigure *event)
{
    
    if(GET_PIX(PIX_BUF_VW)) return TRUE;
    STORE_PIX(PIX_BUF_VW,gdk_pixmap_new(wg->window,PICT_AREA_WIDTH,PICT_AREA_HEIGHT,-1));
    return TRUE;
}

gint expose_pict(GtkWidget *wg,GdkEventExpose *event)
{
    gdk_draw_pixmap(wg->window,
	wg->style->fg_gc[GTK_WIDGET_STATE(wg)],GET_PIX(PIX_BUF_VW),
	event->area.x,event->area.y, event->area.x,event->area.y,	
	event->area.width, event->area.height);
    return FALSE;
}

gint configure_sw(GtkWidget *wg,GdkEventConfigure *event)
{
    if(GET_PIX(PIX_BUF_SW)) return TRUE;
    STORE_PIX(PIX_BUF_SW,gdk_pixmap_new(wg->window,SW_AREA_WIDTH,SW_AREA_HEIGHT,-1));
    return TRUE;
}

gint expose_sw(GtkWidget *wg,GdkEventExpose *event)
{
    gdk_draw_pixmap(wg->window,
	wg->style->fg_gc[GTK_WIDGET_STATE(wg)],GET_PIX(PIX_BUF_SW),
	event->area.x,event->area.y, event->area.x,event->area.y,	
	event->area.width, event->area.height);
    return FALSE;
}

void drawable_rfsh(int id)
{
    GdkRectangle temp;
    temp.x = temp.y = 0;
    temp.width = GET_WG(id)->allocation.width;
    temp.height = GET_WG(id)->allocation.height;
    gtk_widget_draw(GET_WG(id), &temp);
}

void draw_pict()
{
    int pxx,pyy;

    gdk_drawable_get_size(GET_PIX(PIX_BUF_VW),&pxx,&pyy);
    gdk_draw_rectangle(GET_PIX(PIX_BUF_VW),
    GET_WG(WG_WMAIN)->style->bg_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))], TRUE, 0, 0, pxx, pyy);
    if(GET_C_IMAGE)
	gdk_draw_pixmap(GET_PIX(PIX_BUF_VW),
		GET_WG( WG_WMAIN)->style->fg_gc[ GTK_WIDGET_STATE(GET_WG(WG_WMAIN) ) ],
		GET_C_IMAGE,
		0, 0, 0, 0, pxx, pyy);
    drawable_rfsh(WG_VW_DA);
}

void draw_dip_switch()
{
    int i,tmp,sxx,syy,dxx,dyy,allign_x,allign_y;
    PangoLayout *pl=NULL;
    PangoFontDescription *pfd;
    char *sw_dsc[]={SW_DESCRIPTION};

    pfd = pango_font_description_from_string(SW_FONT_DSC);
    gdk_drawable_get_size(GET_PIX(WG_SW_ON),&sxx,&syy);
    gdk_drawable_get_size(GET_PIX(PIX_BUF_SW),&dxx,&dyy);    
    allign_x = dxx - sxx * DIP_SWITCH_SIZE - 5;
    allign_y = dyy - syy;
    gdk_draw_rectangle(GET_PIX(PIX_BUF_SW),
    GET_WG(WG_WMAIN)->style->bg_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))], TRUE, 0, 0, dxx, dyy);
    for(i = 0; i < DIP_SWITCH_SIZE; i++){
	if(i == 1) continue;
	pl = gtk_widget_create_pango_layout(GET_WG(WG_SW_DA),
		sw_dsc[i ? ( GET_PCB_WILLEM ? (DIP_SWITCH_SIZE + 1 - i) : i) : 0]);
	pango_layout_set_font_description(pl, pfd);
	gdk_draw_layout(GET_PIX(PIX_BUF_SW),
		GET_WG(WG_WMAIN)->style->black_gc,allign_x+i*sxx - (i ? 0 : 5), 0, pl);
	tmp=0;
	if( i > 1 )
	    tmp = !GET_PCB_WILLEM ? (GET_DIP_SWITCH << (i-2) ) & 0x800 : 
					    (GET_DIP_SWITCH >> (i-2) ) & 0x001;
	if( i == 0)
	    tmp = GET_DIP_SWITCH & 0x1000;
	    
	gdk_draw_pixmap(GET_PIX(PIX_BUF_SW),
	GET_WG( WG_WMAIN)->style->fg_gc[ GTK_WIDGET_STATE(GET_WG(WG_WMAIN) ) ],
		((GET_PCB_WILLEM && i) ? !tmp : tmp) ? GET_PIX(WG_SW_ON) : GET_PIX(WG_SW_OFF),
		0, 0, allign_x + i * sxx, allign_y, sxx, syy);

    }        
    drawable_rfsh(WG_SW_DA);
    pango_font_description_free(pfd);
    g_object_unref(pl);
}

void set_rd_bt_lp(){
    switch(GET_PORT_ADDR){
	case LP0_ADDR : SET_BT(RD_BUTTON_LP1,0); break;
	case LP1_ADDR : SET_BT(RD_BUTTON_LP1,1); break;
	case LP2_ADDR : SET_BT(RD_BUTTON_LP1,2); break;
    }
}


gint bt_rd_lp(GtkWidget *wg){
    if( GTK_TOGGLE_BUTTON(GET_WG(RD_BUTTON_LP0))->active ){
	SET_PORT_ADDR(LP0_ADDR);    
    } else 
    if( GTK_TOGGLE_BUTTON(GET_WG(RD_BUTTON_LP1))->active ){
	SET_PORT_ADDR(LP1_ADDR);    
    } else
     if( GTK_TOGGLE_BUTTON(GET_WG(RD_BUTTON_LP2))->active ){
	SET_PORT_ADDR(LP2_ADDR);    
    }
    stat_rfsh(); 
    init_ports();
    reset_ports();
    return TRUE;
}

gint bt_willem_ev(GtkWidget *wg)
{
    SET_PCB_WILLEM(GTK_TOGGLE_BUTTON(GET_WG(CH_BT_PCB))->active);
    stat_rfsh();
    return TRUE;
}


gint expose_view(GtkWidget *wg,GdkEventExpose *event)
{
    gdk_draw_pixmap(wg->window,
	wg->style->fg_gc[GTK_WIDGET_STATE(wg)],GET_PIX(PIX_BUF_VIEW),
	event->area.x,event->area.y, event->area.x,event->area.y,	
	event->area.width, event->area.height);
    return FALSE;

}

gint configure_view(GtkWidget *wg,GdkEventExpose *event)
{
    if(GET_PIX(PIX_BUF_VIEW)) return TRUE;
    STORE_PIX(PIX_BUF_VIEW,gdk_pixmap_new(wg->window,wg->allocation.width,wg->allocation.height,-1));
    return TRUE;
}


char dig2hex(char i){
    return i + (i < 10 ? '0'  : 'A' - 10 );
}

char ascii_filter(unsigned char i){
    if((i<33)||(i>127)) i='.';
    return i;
}

void set_viewer_size(int size){
    GTK_ADJUSTMENT(GET_ADJ(WG_VIEWER_ADJ))->value=0;
    GTK_ADJUSTMENT(GET_ADJ(WG_VIEWER_ADJ))->lower=0;
    GTK_ADJUSTMENT(GET_ADJ(WG_VIEWER_ADJ))->upper=size;
    GTK_ADJUSTMENT(GET_ADJ(WG_VIEWER_ADJ))->step_increment=16;
    GTK_ADJUSTMENT(GET_ADJ(WG_VIEWER_ADJ))->page_increment=16*16;
    GTK_ADJUSTMENT(GET_ADJ(WG_VIEWER_ADJ))->page_size=16*21;
    gtk_signal_emit_by_name(GTK_OBJECT(GET_ADJ(WG_VIEWER_ADJ)),"changed");
}

void viewer_slider(GtkAdjustment *adj){
    int a;
    a = adj->value;
    SET_BF_SH(a & ~0x0fL);
    viewer_rfsh();
}


char *bin2hex(char *str,int value, int length){
    int i;

    *( str += length) = 0;
    for(i = 0; i < length; i++, value >>= 4 ) 
	*(--str) = dig2hex(value & 0x0f);    
    return str;
}

void viewer_rfsh()
{
    int vxx,vyy,i,j;
    PangoLayout *pl=NULL;
    PangoFontDescription *pfd;
    char tmp[17];
    int  address;

    address=GET_BF_SH;
    gdk_drawable_get_size(GET_PIX(PIX_BUF_VIEW),&vxx,&vyy);
    gdk_draw_rectangle(GET_PIX(PIX_BUF_VIEW),
    GET_WG(WG_WMAIN)->style->white_gc, TRUE, 0, 0, vxx, vyy);
    gdk_draw_rectangle(GET_PIX(PIX_BUF_VIEW),
    GET_WG(WG_WMAIN)->style->bg_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))], 
    TRUE, 0, 0, vxx, CELL_TITLE_Y);
    gdk_draw_rectangle(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->white_gc, 
    FALSE, 0, 0, vxx, CELL_TITLE_Y);
    gdk_draw_rectangle(GET_PIX(PIX_BUF_VIEW),
    GET_WG(WG_WMAIN)->style->light_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))], 
    FALSE, 1, 1, vxx-1, CELL_TITLE_Y-1);
    gdk_draw_line(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
    1,CELL_TITLE_Y,vxx,CELL_TITLE_Y);
    gdk_draw_line(GET_PIX(PIX_BUF_VIEW),
    GET_WG(WG_WMAIN)->style->dark_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))],
    1,CELL_TITLE_Y-1,vxx,CELL_TITLE_Y-1);
    gdk_draw_line(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
    vxx-1,0,vxx-1,CELL_TITLE_Y);    
    gdk_draw_line(GET_PIX(PIX_BUF_VIEW),
    GET_WG(WG_WMAIN)->style->dark_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))],
    vxx-2,0,vxx-2,CELL_TITLE_Y);

    pfd = pango_font_description_from_string(VIEW_FONT_DSC);
    pl = gtk_widget_create_pango_layout(GET_WG(WG_VIEW_DA),"adres");
    pango_layout_set_font_description(pl, pfd);
    gdk_draw_layout(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,10, 4, pl);
    g_object_unref(pl);

    pl = gtk_widget_create_pango_layout(GET_WG(WG_VIEW_DA),"ASCII");
    pango_layout_set_font_description(pl, pfd);
    gdk_draw_layout(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,370, 4, pl);
    g_object_unref(pl);

    for(i = 0; i < 17; i++){
	gdk_draw_line(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
	  CELL_ADRESS_SIZE_X + i * CELL_SIZE_X, 0, CELL_ADRESS_SIZE_X + i * CELL_SIZE_X, vyy);
	gdk_draw_line(GET_PIX(PIX_BUF_VIEW),
	GET_WG(WG_WMAIN)->style->dark_gc[GTK_WIDGET_STATE(GET_WG(WG_WMAIN))],
	  CELL_ADRESS_SIZE_X + i * CELL_SIZE_X-1,0,CELL_ADRESS_SIZE_X + i * CELL_SIZE_X - 1, CELL_TITLE_Y);
	gdk_draw_line(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->white_gc,
	  CELL_ADRESS_SIZE_X + i * CELL_SIZE_X + 1, 0, CELL_ADRESS_SIZE_X + i * CELL_SIZE_X + 1, CELL_TITLE_Y);
	gdk_draw_line(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->white_gc,
	  CELL_ADRESS_SIZE_X+i*CELL_SIZE_X+2,0,CELL_ADRESS_SIZE_X+i*CELL_SIZE_X+2,CELL_TITLE_Y);
	if(i){
	    pl = gtk_widget_create_pango_layout(GET_WG(WG_VIEW_DA),bin2hex(tmp,i-1,2));
	    pango_layout_set_font_description(pl, pfd);
	    gdk_draw_layout(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
	    CELL_ADRESS_SIZE_X + (i-1)*CELL_SIZE_X + 2, 4, pl);
	    g_object_unref(pl);
	}
    }
    for(i=0;i < (vyy / CELL_SIZE_Y);i++){
	gdk_draw_line(GET_PIX(PIX_BUF_VIEW), GET_WG(WG_WMAIN)->style->black_gc,
	  0, CELL_TITLE_Y + i * CELL_SIZE_Y, vxx, CELL_TITLE_Y + i * CELL_SIZE_Y);
    }
    for(j=0;(j < (vyy / CELL_SIZE_Y)-7) && (address < GET_DEV_SIZE);j++){
	pl = gtk_widget_create_pango_layout(GET_WG(WG_VIEW_DA),bin2hex(tmp,address,8));
    	pango_layout_set_font_description(pl, pfd);
	gdk_draw_layout(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
	2, j * CELL_SIZE_Y+2+CELL_TITLE_Y, pl);
	g_object_unref(pl);
        for(i = 0; (i < 16) && (address < GET_DEV_SIZE); i++){
	    *(tmp+0) = ascii_filter(BUFFER(address));
	    *(tmp+1) = 0;
	    pl = gtk_widget_create_pango_layout(GET_WG(WG_VIEW_DA),tmp);
    	    pango_layout_set_font_description(pl, pfd);
	    gdk_draw_layout(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
	    360+i*6, j*CELL_SIZE_Y+2+CELL_TITLE_Y, pl);
	    g_object_unref(pl);
	    pl = gtk_widget_create_pango_layout(GET_WG(WG_VIEW_DA),bin2hex(tmp,BUFFER(address),2));
	    pango_layout_set_font_description(pl, pfd);
	    gdk_draw_layout(GET_PIX(PIX_BUF_VIEW),GET_WG(WG_WMAIN)->style->black_gc,
	    CELL_ADRESS_SIZE_X + i*CELL_SIZE_X + 2, j*CELL_SIZE_Y+2+CELL_TITLE_Y, pl);
	    g_object_unref(pl);
	    address++;
	}
    }
    drawable_rfsh(WG_VIEW_DA);
    pango_font_description_free(pfd);
}

void device_menu_create(GtkWidget *wg)
{
    chip *chp;
    char *tmp, t;
    void *p,*op;
    GtkWidget *grp, *tt;

    add_new_path_qe("/",wg);
    for(reset_queue_chip(); (chp = get_chip_qe()); next_chip() ){

	tmp = chp->chip_path + 1;
	op = find_path_qe("/");
	for(;*tmp; tmp++){
	    for(; *tmp!='/' && *tmp; tmp++);
	    t = *tmp;
	    *tmp = 0;
	    if(!(p = find_path_qe(chp->chip_path))){
		grp = gtk_menu_item_new_with_label(last_pth(chp->chip_path));
		gtk_menu_bar_append(GTK_MENU(op), grp);
		p = gtk_menu_new();
 		gtk_menu_item_set_submenu(GTK_MENU_ITEM(grp), p);
		add_new_path_qe(chp->chip_path, p);
	    }
	    op = p;
	    *tmp = t;
	}

	if((p = find_path_qe(chp->chip_path))){
	    tt = gtk_menu_item_new_with_label(chp->chip_name);
	    gtk_menu_append(GTK_MENU(p), tt);
	    gtk_signal_connect(GTK_OBJECT(tt), "activate", GTK_SIGNAL_FUNC(device_sel), chp->chip_name );
	}
    }
}

void add_action(int idx, int rm_idx, char *name, gchar *tip, void *fnc, void *prm)
{
    GtkWidget *tmp;
    tmp = gtk_image_new_from_stock(name, gtk_toolbar_get_icon_size(GTK_TOOLBAR(GET_WG(TB_CONTAINER))));    
    STORE_WG(rm_idx,
	gtk_toolbar_append_item(GTK_TOOLBAR(GET_WG(TB_CONTAINER)), NULL, tip, NULL, tmp, fnc, prm));
    STORE_WG(idx, tmp);
}

void fast_option_add(int type, char *name, int value, void *fnc, void *parm, char pos){
    dynamic_widget_add(&fo_mg, &fo_mg_first, FAST_OPTION_MENU, type, name, value, fnc, parm, pos);
}

void adv_option_add(int type, char *name, int value, void *fnc, void *parm, char pos)
{
    dynamic_widget_add(&ao_mg, &ao_mg_first, ADV_OPTION, type, name, value, fnc, parm, pos);
}

void adv_option_purge()
{
    dynamic_widget_purge(&ao_mg, &ao_mg_first);
    for(; fo_cpage - 3; fo_cpage--)
	gtk_notebook_remove_page(GTK_NOTEBOOK(GET_WG(NB_MAIN)), fo_cpage - 1);
}

void fast_option_purge()
{
    dynamic_widget_purge(&fo_mg, &fo_mg_first);
}

void adv_add_page(char *name)
{
    GtkWidget *t_0,*t_1;

    t_0 = gtk_vbox_new(FALSE,0);
    t_1 = gtk_label_new(name);
    gtk_notebook_append_page(GTK_NOTEBOOK(GET_WG(NB_MAIN)), t_0, t_1);
    t_1 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(t_0),t_1);
    STORE_WG(ADV_OPTION, t_1);
    gtk_widget_show_all(GET_WG(NB_MAIN));
    fo_cpage++;
}

void dynamic_widget_add(fo_idx **fo, fo_idx **ff ,int idx ,int type, char *name, int value, void *fnc, void *parm, char pos)
{
	static GSList *grp;
	static GtkWidget *hbox;
	char	t[256];
	GtkWidget *tmp;
	fo_idx	*temp;
	
	if(!(temp = (fo_idx *)malloc(sizeof(fo_idx)))){
	    perror(MALLOC_ERROR);
	    return;
	}
	temp->next = NULL;
	
	switch(type){
	    case FO_LABEL : 	temp->wg  = gtk_label_new(name);
				break;
	    case FO_RB_FIRST: 	grp = NULL;
	    case FO_RB_NEXT:  	temp->wg = gtk_radio_button_new_with_label(grp, name);
				grp = gtk_radio_button_group(GTK_RADIO_BUTTON(temp->wg));
				break;
	    case FO_CHECK : 	temp->wg = gtk_check_button_new_with_label(name);
				break;
	    case FO_ENTRY : 	temp->wg = gtk_hbox_new(FALSE,0);
				tmp = gtk_label_new(name);
				gtk_widget_show(tmp);
				gtk_box_pack_start(GTK_BOX(temp->wg),tmp,FALSE,FALSE,0);
				tmp = gtk_entry_new();
				sprintf(t,"0x%x",value);
				gtk_entry_set_text(GTK_ENTRY(tmp), t);
				gtk_entry_set_editable(GTK_ENTRY(tmp),TRUE);
				gtk_widget_show(tmp);
				gtk_box_pack_start(GTK_BOX(temp->wg),tmp,FALSE,FALSE,0);
				break;			    
	    default: free(temp); return;
	}
	gtk_widget_show(temp->wg);
	switch(pos){
	    case FO_H_FIRST: hbox = gtk_hbox_new(FALSE,0);
			     gtk_box_pack_start(GTK_BOX(hbox),temp->wg,FALSE,FALSE,0);
			     gtk_widget_show(hbox);
			     gtk_box_pack_start(GTK_BOX(GET_WG(idx)),hbox,FALSE,FALSE,0);
			     break;
	    case FO_H_NEXT:  gtk_box_pack_start(GTK_BOX(hbox),temp->wg,FALSE,FALSE,0);
			     break;
	    case FO_NONE:    gtk_box_pack_start(GTK_BOX(GET_WG(idx)),temp->wg,FALSE,FALSE,0);
			     break;
	    default: free(temp); return;

	}

	if(fnc)		
	    gtk_signal_connect(GTK_OBJECT(temp->wg),
			       "clicked",GTK_SIGNAL_FUNC(fnc), parm);

	if(!*ff){
	    *fo = *ff = temp;
	    return;
	}
	for(*fo = *ff; (*fo)->next; *fo = (*fo)->next);
	(*fo)->next = temp;
	*fo = *fo;
}

void dynamic_widget_purge(fo_idx **fo, fo_idx **ff)
{
    fo_idx *tmp;
    for(*fo = *ff; *fo;){
	tmp = (*fo)->next;
	gtk_widget_destroy( (*fo)->wg);
	free(*fo);
	*fo = tmp;
    }
    *fo = *ff = NULL;
}

void chip_pin(GtkWidget *wg, char *pin_name)
{
    int addr,dta, i,tmp;
    char *tab_cmp[34] = { 
	PIN_01,PIN_02,PIN_03,PIN_04,PIN_05,PIN_06,PIN_07,PIN_08,PIN_09,PIN_10,PIN_11,PIN_12,PIN_13,
	PIN_14,PIN_15,PIN_16,PIN_17,PIN_18,PIN_19,PIN_20,PIN_21,PIN_22,PIN_23,PIN_24,PIN_25,PIN_26,
	PIN_27,PIN_28,PIN_29,PIN_30,PIN_31,PIN_32,TST_LB_ADDRESS,TST_LB_DATA };
    char addr_bit2pin[18] = {11,10,9,8,7,6,5,4,26,25,22,24,3,27,28,2,1,29};
    char dta_bit2pin[8] = {12,13,14,16,17,18,19,20};
    
    static char ex_flag=0;
    
    if(ex_flag) return; /* blokada przed rekurencyjnym wywolaniem sygnalu */
    if(GET_STARTAP) return;
    ex_flag = 1;
    
    for(i = 0; i < sizeof(tab_cmp) && strcmp(pin_name, tab_cmp[i]); i++);
    
    addr = dta = -1;

    if(i == 32){
	tmp = gtk_spin_button_get_value(GTK_SPIN_BUTTON(GET_WG(TST_ADDR_BT)));
	for(i = 0; i < 18; i++)
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tst_chkbx[(int)addr_bit2pin[i]]),((0x01 << i) & tmp));
	ex_flag=0;
	set_par_addr_pin(tmp);
	return;
    }
    if(i == 33){
	tmp = gtk_spin_button_get_value(GTK_SPIN_BUTTON(GET_WG(TST_DATA_BT)));
	tmp = gtk_spin_button_get_value(GTK_SPIN_BUTTON(GET_WG(TST_DATA_BT)));
	for(i = 0; i < 8; i++)
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tst_chkbx[(int)dta_bit2pin[i]]),((0x01 << i) & tmp));
	ex_flag=0;
	set_par_data_pin(tmp);
	return;
    }

    if((i > 0) && (i < 3)) addr = 17 - i; if((i > 3) && (i < 12)) addr = 11 - i ;
    if(i == 3) addr = 12; if(i == 29) addr = 17; if(i == 28) addr = 14; if(i == 27) addr = 13;
    if(i == 26) addr = 8; if(i == 25) addr = 9; if(i == 24) addr = 11; if(i == 22) addr = 10;
    if((i > 11) && (i < 16)) dta = i - 12;
    if((i > 15) && (i < 21)) dta = i - 13;

    if(addr >=0 ){
	addr = 0x01 << addr;
	tmp = gtk_spin_button_get_value(GTK_SPIN_BUTTON(GET_WG(TST_ADDR_BT)));
	tmp ^= addr;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(GET_WG(TST_ADDR_BT)),tmp);
	set_par_addr_pin(tmp);
    }
    if(dta >=0 ){
	dta = 0x01 << dta;
	tmp = gtk_spin_button_get_value(GTK_SPIN_BUTTON(GET_WG(TST_DATA_BT)));
	tmp ^= dta;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(GET_WG(TST_DATA_BT)),tmp);
	set_par_data_pin(tmp);
    }
    ex_flag=0;
    
    switch(i){
	case 0:  if(GTK_TOGGLE_BUTTON(wg)->active) vpp_on(); else vpp_off(); break;
	case 21: set_ce_pin(GTK_TOGGLE_BUTTON(wg)->active);break;
	case 23: set_oe_pin(GTK_TOGGLE_BUTTON(wg)->active);break;
	case 30: set_we_pin(GTK_TOGGLE_BUTTON(wg)->active);break;
	case 31: if(GTK_TOGGLE_BUTTON(wg)->active) vcc_on(); else vcc_off(); break;
    }
}

void add_pin(GtkWidget *tmp, char *txt, char rl, float scale, int id)
{
    GtkWidget *tmp_2, * tmp_3, *tmp_4, *tmp_1;
    GtkStyle *style;    
    long fs;

    tmp_1 = gtk_alignment_new( rl ? 1:0, 0.5,0,0);
    gtk_container_add(GTK_CONTAINER(tmp),tmp_1);
    tmp_2 = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_1),tmp_2);
    tmp_3 = gtk_check_button_new();
    tst_chkbx[id] = tmp_3;
    gtk_signal_connect(GTK_OBJECT(tmp_3),"toggled",GTK_SIGNAL_FUNC(chip_pin),txt);
    tmp_4 = gtk_label_new(txt);
    style = gtk_widget_get_style(tmp_4);
    fs = pango_font_description_get_size(style->font_desc);
    pango_font_description_set_size(style->font_desc, fs / scale);
    gtk_widget_modify_font(tmp_4,style->font_desc);
    pango_font_description_set_size(style->font_desc, fs );
    if(!strcmp(txt,PIN_16)) 
	gtk_widget_set_sensitive(GTK_WIDGET(tmp_3), 0);
    if(rl){
	tmp_1 = tmp_4;
	tmp_4 = tmp_3;
	tmp_3 = tmp_1;
    }
    gtk_box_pack_start(GTK_BOX(tmp_2), tmp_3, FALSE,FALSE, 0); 
    gtk_box_pack_start(GTK_BOX(tmp_2), tmp_4, FALSE,FALSE, 0); 

}

void rd_test_data()
{
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(GET_WG(TST_DATA_BT)), get_par_data_pin());
}

void serial_test(GtkWidget *wg, gpointer f_name){
    char tmp_str[16];
    
    if(!strcmp(f_name,"RD")){
	sprintf(tmp_str,"%i",get_do_pin()); 
	gtk_entry_set_text(GTK_ENTRY(GET_WG(TST_SERIAL_ENTRY)),tmp_str);
    }
    if(!strcmp(f_name,"DI")) set_di_pin(GTK_TOGGLE_BUTTON(wg)->active);
    if(!strcmp(f_name,"CLK")) set_clk_pin(GTK_TOGGLE_BUTTON(wg)->active);
    if(!strcmp(f_name,"CS")) set_cs_pin(GTK_TOGGLE_BUTTON(wg)->active);
}

void programmer_test()
{
    GtkWidget *hbox, *tmp_2, *tmp_0, *tmp_1, *tmp, *tmp_3;
    GdkBitmap *mask;
    GdkPixmap *pix;
    GtkAdjustment *adj;    

    hbox = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(GET_WG(NB_TEST_HW)), hbox);

/* pierwsza ramka */
    tmp_3 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(hbox), tmp_3);    

    tmp_0 = gtk_frame_new(TST_LB_ZIF);
    gtk_frame_set_shadow_type(GTK_FRAME(tmp_0), GTK_SHADOW_OUT);
    gtk_frame_set_label_align(GTK_FRAME(tmp_0), 0.5,0);
    gtk_container_add(GTK_CONTAINER(tmp_3), tmp_0);
    tmp_1 = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_0), tmp_1);    
    
    tmp_2 = gtk_frame_new(NULL);    
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_2);    

    tmp = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_2), tmp);        

    ADD_PIN_L(PIN_01,0);  ADD_PIN_L(PIN_02,1);  ADD_PIN_L(PIN_03,2);  ADD_PIN_L(PIN_04,3);
    ADD_PIN_L(PIN_05,4);  ADD_PIN_L(PIN_06,5);  ADD_PIN_L(PIN_07,6);  ADD_PIN_L(PIN_08,7);
    ADD_PIN_L(PIN_09,8);  ADD_PIN_L(PIN_10,9);  ADD_PIN_L(PIN_11,10); ADD_PIN_L(PIN_12,11);
    ADD_PIN_L(PIN_13,12); ADD_PIN_L(PIN_14,13); ADD_PIN_L(PIN_15,14); ADD_PIN_L(PIN_16,15);	

    tmp_2 = gtk_frame_new(NULL);    
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_2);    
    tmp = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_2), tmp);        

    ADD_PIN_R(PIN_32,31); ADD_PIN_R(PIN_31,30); ADD_PIN_R(PIN_30,29); ADD_PIN_R(PIN_29,28);
    ADD_PIN_R(PIN_28,27); ADD_PIN_R(PIN_27,26); ADD_PIN_R(PIN_26,25); ADD_PIN_R(PIN_25,24);
    ADD_PIN_R(PIN_24,23); ADD_PIN_R(PIN_23,22); ADD_PIN_R(PIN_22,21); ADD_PIN_R(PIN_21,20);
    ADD_PIN_R(PIN_20,19); ADD_PIN_R(PIN_19,18); ADD_PIN_R(PIN_18,17); ADD_PIN_R(PIN_17,16);


/* druga ramka */
    tmp_2 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(hbox), tmp_2);

    tmp_0 = gtk_frame_new(TST_LB_SET_AD);
    gtk_frame_set_label_align(GTK_FRAME(tmp_0), 0.5, 0.5);
    gtk_box_pack_start(GTK_BOX(tmp_2), tmp_0, FALSE, FALSE, 5);
    tmp = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_0), tmp);    

    tmp_1 = gtk_hbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(tmp), tmp_1, FALSE,FALSE,5);
    tmp_0 = gtk_label_new(TST_LB_ADDRESS);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);
    adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0.0, 0x3ffff, 1, 16, 16));
    tmp_0 = gtk_spin_button_new(adj, 1, 0);
    gtk_widget_set_usize(tmp_0,50,25);
    gtk_signal_connect(GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(chip_pin), TST_LB_ADDRESS);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);
    STORE_WG(TST_ADDR_BT,tmp_0);    

    tmp_1 = gtk_hbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(tmp), tmp_1, FALSE,FALSE,5);
    tmp_0 = gtk_label_new(TST_LB_DATA);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);
    adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0.0, 0xff, 1, 16, 16));
    gtk_signal_connect(GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(chip_pin), TST_LB_DATA);
    tmp_0 = gtk_spin_button_new(adj, 1, 0);
    gtk_widget_set_usize(tmp_0,50,25);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);
    STORE_WG(TST_DATA_BT,tmp_0);    
    tmp_1 = gtk_button_new_with_label(TST_LB_RD_DATA);
    gtk_signal_connect(GTK_OBJECT(tmp_1),"clicked",GTK_SIGNAL_FUNC(rd_test_data),NULL);
    gtk_container_add(GTK_CONTAINER(tmp), tmp_1);

    tmp_1 = gtk_frame_new(TST_LB_SER_FR);
    gtk_frame_set_label_align(GTK_FRAME(tmp_1), 0.5, 0.5);
    gtk_box_pack_start(GTK_BOX(tmp_2), tmp_1, FALSE, FALSE, 5);
    tmp_0 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);

    tmp_1 = gtk_check_button_new_with_label(TST_LB_CS);
    gtk_signal_connect(GTK_OBJECT(tmp_1),"clicked",GTK_SIGNAL_FUNC(serial_test),"CS");
    gtk_box_pack_start(GTK_BOX(tmp_0), tmp_1, FALSE, FALSE, 5);

    tmp_1 = gtk_check_button_new_with_label(TST_LB_CLK);
    gtk_signal_connect(GTK_OBJECT(tmp_1),"clicked",GTK_SIGNAL_FUNC(serial_test),"CLK");
    gtk_box_pack_start(GTK_BOX(tmp_0), tmp_1, FALSE, FALSE, 5);

    tmp_1 = gtk_check_button_new_with_label(TST_LB_DI);
    gtk_signal_connect(GTK_OBJECT(tmp_1),"clicked",GTK_SIGNAL_FUNC(serial_test),"DI");
    gtk_box_pack_start(GTK_BOX(tmp_0), tmp_1, FALSE, FALSE, 5);

    tmp_1 = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_0), tmp_1);
    tmp_0 = gtk_button_new_with_label(TST_LB_SER_RD);
    gtk_signal_connect(GTK_OBJECT(tmp_0),"clicked",GTK_SIGNAL_FUNC(serial_test),"RD");
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);

    tmp_0 = gtk_entry_new();
    gtk_entry_set_editable(GTK_ENTRY(tmp_0),FALSE);
    gtk_widget_set_usize(tmp_0,50,25);
    STORE_WG(TST_SERIAL_ENTRY,tmp_0);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp_0);

    tmp_1 = gtk_frame_new(TST_LB_DIP);
    gtk_frame_set_label_align(GTK_FRAME(tmp_1), 0.5, 0.5);
    gtk_box_pack_start(GTK_BOX(tmp_2), tmp_1, TRUE, TRUE, 5);

    STORE_WG(SWITCH_TEST_HW_BX, tmp_1);
    pix = gdk_pixmap_create_from_xpm_d(GET_WG(MAIN_WINDOW)->window,&mask,NULL,
			GET_PCB_WILLEM ? (gchar **)SW_TEST_WILLEM_XPM : (gchar **)SW_TEST_PCB3_XPM);
    tmp = gtk_pixmap_new(pix,mask);
    STORE_WG(SWITCH_TEST_HW, tmp);
    gtk_container_add(GTK_CONTAINER(tmp_1), tmp);


}

void menu_setup(void)
{
    GSList *algo_grp=NULL;
    GtkWidget *wmain,*vfield_1,*nb,*wg_temp_0,*wg_temp_1,*wg_temp_2,*wg_temp_3;
    GdkPixmap *pix;
    GdkBitmap *mask;
    GtkAdjustment *adj;

    gtk_init(NULL,NULL);

    wmain=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    STORE_WG(MAIN_WINDOW,wmain);
    gtk_signal_connect(GTK_OBJECT(wmain),"delete_event",GTK_SIGNAL_FUNC(kill_me),NULL);
    gtk_container_border_width(GTK_CONTAINER(wmain),1);
    gtk_window_set_title(GTK_WINDOW(wmain),"geepro v-0.0.1");
    gtk_widget_set_usize(wmain,MAIN_WINDOW_WIDTH,MAIN_WINDOW_HEIGHT);
//    gtk_window_set_policy(GTK_WINDOW(wmain),FALSE,FALSE,TRUE);
    gtk_widget_show(wmain);
    vfield_1=gtk_vbox_new(FALSE, 0);    
    gtk_container_add(GTK_CONTAINER(wmain), vfield_1);

    wg_temp_1 = gtk_menu_bar_new();
    gtk_box_pack_start(GTK_BOX(vfield_1),wg_temp_1,FALSE,FALSE,0);
/* menu bar*/
    wg_temp_2 = gtk_menu_item_new_with_label(MB_FILE);
    gtk_menu_bar_append(GTK_MENU_BAR(wg_temp_1), wg_temp_2);
    wg_temp_0 = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(wg_temp_2),wg_temp_0);

/* File */
    wg_temp_2 = gtk_menu_item_new_with_label(MB_LOAD_BIN_FILE);
    gtk_menu_append(GTK_MENU(wg_temp_0), wg_temp_2);    
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"activate",GTK_SIGNAL_FUNC(load_file),NULL);
    wg_temp_2 = gtk_menu_item_new_with_label(MB_SAVE_BIN_FILE);
    gtk_menu_append(GTK_MENU(wg_temp_0), wg_temp_2);    
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"activate",GTK_SIGNAL_FUNC(save_file),NULL);
    wg_temp_2 = gtk_menu_item_new();
    gtk_menu_append(GTK_MENU(wg_temp_0), wg_temp_2);    
    wg_temp_2 = gtk_menu_item_new_with_label(MB_ABOUT_FILE);
    gtk_menu_append(GTK_MENU(wg_temp_0), wg_temp_2);    
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"activate",GTK_SIGNAL_FUNC(about),NULL);
    wg_temp_2 = gtk_menu_item_new_with_label(MB_EXIT_FILE);
    gtk_menu_append(GTK_MENU(wg_temp_0), wg_temp_2);    
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"activate",GTK_SIGNAL_FUNC(exit_program),NULL);

/* Menu device */
    wg_temp_2 = gtk_menu_item_new_with_label(MB_DEVICE);    
    gtk_menu_bar_append(GTK_MENU_BAR(wg_temp_1), wg_temp_2);
    wg_temp_0 = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(wg_temp_2),wg_temp_0);
    STORE_WG(MB_P_DEVICE,wg_temp_0);

/* toolbar */
    wg_temp_1=gtk_toolbar_new(TOOLBAR_PARAMS);
    gtk_box_pack_start(GTK_BOX(vfield_1),wg_temp_1,FALSE,FALSE,0);
    STORE_WG(TB_CONTAINER, wg_temp_1);
    add_action(TB_P_OPEN_FILE,    0, GTK_STOCK_OPEN, OPEN_FILE_TIP, TB_OPEN_FILE );
    add_action(TB_P_SAVE_FILE,    0, GTK_STOCK_SAVE, SAVE_FILE_TIP, TB_SAVE_FILE );
    add_action(TB_P_CLEAR_BUFFER, 0, GTK_STOCK_CLEAR, CLEAR_BUFFER_TIP, TB_CLEAR_BUFFER );
    gtk_toolbar_append_space(GTK_TOOLBAR(GET_WG(TB_CONTAINER)));

/*notebook*/
    nb = gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(nb),GTK_POS_BOTTOM);
/* notebook page 1 */
    wg_temp_0 = gtk_vbox_new(FALSE,0);
    wg_temp_1 = gtk_label_new(LAB_NOTE_1);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb),wg_temp_0,wg_temp_1);
    wg_temp_3 = gtk_table_new(3,3,FALSE);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_3);
    wg_temp_1 = gtk_frame_new(FR_NB_01_TITLE);
    gtk_table_attach_defaults(GTK_TABLE(wg_temp_3),wg_temp_1,0,1,0,1);
    wg_temp_0 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_0);
    wg_temp_1 = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);

    wg_temp_2 = gtk_label_new(DEVICE_ENTRY_LB);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);    
    wg_temp_2 = gtk_entry_new();
    STORE_WG(EN_DEV_NAME,wg_temp_2);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);    
    gtk_entry_set_editable(GTK_ENTRY(wg_temp_2), FALSE);
    gtk_widget_set_usize(wg_temp_2,60,20);
    wg_temp_1 = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);
    wg_temp_2 = gtk_label_new(SIZE_HEX_LB);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    wg_temp_2 = gtk_entry_new();
    gtk_entry_set_editable(GTK_ENTRY(wg_temp_2), FALSE);
    gtk_widget_set_usize(wg_temp_2,60,20);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);    
    STORE_WG(EN_SIZE_HEX,wg_temp_2);
    wg_temp_1 = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);
    wg_temp_2 = gtk_label_new(CHECKSUM_LB);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    wg_temp_2 = gtk_entry_new();
    gtk_entry_set_editable(GTK_ENTRY(wg_temp_2), FALSE);
    gtk_widget_set_usize(wg_temp_2,60,20);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);    
    STORE_WG(EN_CHECKSUM,wg_temp_2);
    wg_temp_1=gtk_alignment_new(0.5,0.5,0,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);
    STORE_PIX(PIX_BUF_SW,NULL);
    wg_temp_2 = gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(wg_temp_2),SW_AREA_WIDTH,SW_AREA_HEIGHT);
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"expose_event",(GtkSignalFunc)expose_sw, NULL);
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"configure_event",(GtkSignalFunc)configure_sw, NULL);
    gtk_container_add(GTK_CONTAINER(wg_temp_1), wg_temp_2);
    STORE_WG(WG_SW_DA,wg_temp_2);
    configure_sw(wmain,NULL);
    STORE_FONT(FONT_SW,gdk_font_load("fixed"));
    STORE_WG(WG_WMAIN,wmain);
    pix = gdk_pixmap_create_from_xpm_d(wmain->window,&mask,NULL,(gchar **)SW_ON_XPM);
    STORE_WG(WG_SW_ON, (void *)pix);    
    pix = gdk_pixmap_create_from_xpm_d(wmain->window,&mask,NULL,(gchar **)SW_OFF_XPM);
    STORE_WG(WG_SW_OFF, (void *)pix);    

    wg_temp_1 = gtk_frame_new(FR_NB_02_TITLE);
    gtk_table_attach_defaults(GTK_TABLE(wg_temp_3),wg_temp_1,2,3,0,1);

    wg_temp_0 = gtk_alignment_new(0.5,0.5,0,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_0);
    STORE_PIX(PIX_BUF_VW,NULL);
    wg_temp_1 = gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(wg_temp_1),PICT_AREA_WIDTH,PICT_AREA_HEIGHT);
    gtk_signal_connect(GTK_OBJECT(wg_temp_1),"expose_event",(GtkSignalFunc)expose_pict, NULL);
    gtk_signal_connect(GTK_OBJECT(wg_temp_1),"configure_event",(GtkSignalFunc)configure_pict, NULL);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);
    configure_pict(wmain,NULL);
    STORE_WG(WG_VW_DA,wg_temp_1);

    wg_temp_1 = gtk_frame_new(LB_FR_FAST_OPTION);
    gtk_table_attach_defaults(GTK_TABLE(wg_temp_3),wg_temp_1,0,1,2,3);
    wg_temp_2 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    STORE_WG(FAST_OPTION_MENU, wg_temp_2);

    wg_temp_0 = gtk_frame_new(FR_NB_04_TITLE);
    gtk_table_attach_defaults(GTK_TABLE(wg_temp_3),wg_temp_0,2,3,2,3);
    wg_temp_1 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);

    wg_temp_2 = gtk_check_button_new_with_label(PCB_WILLEM_LB);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"clicked",GTK_SIGNAL_FUNC(bt_willem_ev),NULL);
    STORE_WG(CH_BT_PCB,wg_temp_2);

    wg_temp_2 = gtk_radio_button_new_with_label(algo_grp,LP0_LB);
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"clicked",GTK_SIGNAL_FUNC(bt_rd_lp),NULL);
    algo_grp=gtk_radio_button_group(GTK_RADIO_BUTTON(wg_temp_2));
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    STORE_WG(RD_BUTTON_LP0,wg_temp_2);
    wg_temp_2 = gtk_radio_button_new_with_label(algo_grp,LP1_LB);
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"clicked",GTK_SIGNAL_FUNC(bt_rd_lp),NULL);
    algo_grp=gtk_radio_button_group(GTK_RADIO_BUTTON(wg_temp_2));
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    STORE_WG(RD_BUTTON_LP1,wg_temp_2);
    wg_temp_2 = gtk_radio_button_new_with_label(algo_grp,LP2_LB);
    gtk_signal_connect(GTK_OBJECT(wg_temp_2),"clicked",GTK_SIGNAL_FUNC(bt_rd_lp),NULL);
    algo_grp=gtk_radio_button_group(GTK_RADIO_BUTTON(wg_temp_2));
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    STORE_WG(RD_BUTTON_LP2,wg_temp_2);
    wg_temp_0=gtk_button_new_with_label(TEST_BUTTON_LB);
    gtk_container_border_width(GTK_CONTAINER(wg_temp_0),10);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_0);
    gtk_signal_connect(GTK_OBJECT(wg_temp_0),"clicked",GTK_SIGNAL_FUNC(test_hw),"1");	

/* page 2 */
    wg_temp_0=gtk_hbox_new(FALSE,0);
    wg_temp_1=gtk_label_new(LAB_NB_BUFFER);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb),wg_temp_0,wg_temp_1);
    STORE_WG(NB_BUFFER, wg_temp_0);

/* page 3 */
    wg_temp_0=gtk_hbox_new(FALSE,0);
    wg_temp_1=gtk_label_new(LAB_NOTE_3);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb),wg_temp_0,wg_temp_1);
    STORE_WG(NB_TEST_HW, wg_temp_0);
/* page 4 */
    STORE_WG(NB_MAIN,nb);
    wg_temp_0=gtk_hbox_new(FALSE, 0);    
    gtk_container_add(GTK_CONTAINER(vfield_1), wg_temp_0);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),nb);

/*bufor*/
    STORE_PIX(PIX_BUF_VIEW,NULL);
    wg_temp_1 = gtk_hbox_new(FALSE,0);
/*    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);  */
    gtk_container_add(GTK_CONTAINER(GET_WG(NB_BUFFER)),wg_temp_1);
    wg_temp_2 = gtk_frame_new(NULL);    
    gtk_frame_set_shadow_type(GTK_FRAME(wg_temp_2),GTK_SHADOW_IN);
    wg_temp_3 = gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(wg_temp_3),350-20+70,wg_temp_2->allocation.height);
    gtk_signal_connect(GTK_OBJECT(wg_temp_3),"expose_event",GTK_SIGNAL_FUNC(expose_view),NULL);    
    gtk_signal_connect(GTK_OBJECT(wg_temp_3),"configure_event",GTK_SIGNAL_FUNC(configure_view),NULL);    
    STORE_WG(WG_VIEW_DA,wg_temp_3);
    configure_view(wmain,NULL);
    
    gtk_container_add(GTK_CONTAINER(wg_temp_2),wg_temp_3);
    gtk_container_add(GTK_CONTAINER(wg_temp_1),wg_temp_2);
    
    adj = GTK_ADJUSTMENT(gtk_adjustment_new(0,0,0,16,64,64));
    STORE_ADJ(WG_VIEWER_ADJ,adj);

    gtk_signal_connect(GTK_OBJECT(adj),"value_changed",GTK_SIGNAL_FUNC(viewer_slider),NULL);    
    wg_temp_2 = gtk_vscrollbar_new(adj);
    gtk_box_pack_start(GTK_BOX(wg_temp_1),wg_temp_2,0,0,0);


/* status bar */
    wg_temp_0 = gtk_frame_new(NULL);
    gtk_box_pack_start(GTK_BOX(vfield_1),wg_temp_0,FALSE,FALSE,0);    
    gtk_frame_set_shadow_type(GTK_FRAME(wg_temp_0),GTK_SHADOW_IN);
    wg_temp_1=gtk_label_new(STATUS_BAR_START_TXT);
    gtk_container_add(GTK_CONTAINER(wg_temp_0),wg_temp_1);
    STORE_WG(LB_STATUS_VAL,wg_temp_1);

    fast_option_add(FO_LABEL,NO_FO_MSG, 0 , NULL, NULL,0);
    
    programmer_test();
    set_default();
}

void progress_break()
{
    progress_bar_exit=1;
}

void progress_bar_init(char *title, long range)
{
    GtkWidget *tmp_1, *tmp_2;
    gint x,y,xx,yy;

    gtk_window_get_position(GTK_WINDOW(GET_WG(MAIN_WINDOW)), &x, &y);
    gtk_window_get_size(GTK_WINDOW(GET_WG(MAIN_WINDOW)), &xx, &yy);
    tmp_1 = gtk_window_new(GTK_WINDOW_DIALOG);
    STORE_WG(MPROG_BAR_W,tmp_1);
    gtk_window_move(GTK_WINDOW(tmp_1),x + (xx - xx/2) / 2,y + (yy - PB_HEIGHT) / 2);
    gtk_window_set_modal(GTK_WINDOW(tmp_1),TRUE);
    gtk_signal_connect(GTK_OBJECT(tmp_1), "destroy", GTK_SIGNAL_FUNC(progress_break), NULL);
    gtk_window_set_title(GTK_WINDOW(tmp_1),title);
    gtk_widget_set_usize(tmp_1, xx/2, PB_HEIGHT);
    gtk_widget_realize(tmp_1);
    gdk_window_set_decorations(tmp_1->window, GDK_DECOR_TITLE | GDK_DECOR_BORDER);
    gdk_window_set_functions(tmp_1->window,GDK_FUNC_CLOSE);
    gtk_widget_show(tmp_1);

    tmp_2 = gtk_progress_bar_new_with_adjustment(
    GTK_ADJUSTMENT(gtk_adjustment_new(0,0,range,1,0,0)));
    gtk_container_add(GTK_CONTAINER(tmp_1),tmp_2);
    gtk_widget_show(tmp_2);
    STORE_WG(MPROG_BAR,tmp_2);
    rfsh_gtk();
    progress_bar_exit=0;
}

char progress_bar_set(long value)
{
    if(progress_bar_exit) return 1;
    gtk_progress_set_value(GTK_PROGRESS(GET_WG(MPROG_BAR)), value);
    rfsh_gtk();
    return 0;
}

void progress_bar_free()
{
    if(progress_bar_exit) return;
    gtk_widget_destroy(GET_WG(MPROG_BAR_W));
}

void dialog_bt(GtkWidget *wg, char *name)
{
    if(!strcmp(name,"1")){
	dialog_exit = 1;
	dialog_code = 2;
	return;
    }
    if(!strcmp(name,"0")){
	 dialog_exit = 1;
	 dialog_code = 1;
    }
}

void dialog_kill(){ dialog_exit=1;}

char dialog_box(char *title, char *txt, char *bt1, char *bt2)
{
    GtkWidget *tmp_1, *tmp_2, *tmp_3;
    gint x,y,xx,yy;

    gtk_window_get_position(GTK_WINDOW(GET_WG(MAIN_WINDOW)), &x, &y);
    gtk_window_get_size(GTK_WINDOW(GET_WG(MAIN_WINDOW)), &xx, &yy);
    tmp_1 = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_move(GTK_WINDOW(tmp_1),x + (xx - xx/2) / 2,y + yy / 2);
    gtk_window_set_modal(GTK_WINDOW(tmp_1),TRUE);
    gtk_signal_connect(GTK_OBJECT(tmp_1), "destroy", dialog_kill, NULL);
    gtk_window_set_title(GTK_WINDOW(tmp_1),title);
    gtk_widget_realize(tmp_1);
    gdk_window_set_decorations(tmp_1->window, GDK_DECOR_BORDER | GDK_DECOR_TITLE);
    gdk_window_set_functions(tmp_1->window,GDK_FUNC_CLOSE);
    tmp_2 = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(tmp_1),tmp_2);
    tmp_3 = gtk_label_new(txt);
    gtk_misc_set_padding(GTK_MISC(tmp_3),10,10);
    gtk_container_add(GTK_CONTAINER(tmp_2),tmp_3);
    tmp_3 = gtk_hbox_new(FALSE,0);

    if((bt1)||(bt2))
	gtk_container_add(GTK_CONTAINER(tmp_2),tmp_3);

    if(bt1){
	tmp_2 = gtk_button_new_with_label(bt1);
	gtk_container_add(GTK_CONTAINER(tmp_3),tmp_2);
	gtk_signal_connect(GTK_OBJECT(tmp_2), "clicked", GTK_SIGNAL_FUNC(dialog_bt), "0");
    }

    if(bt2){
	tmp_2 = gtk_button_new_with_label(bt2);
	gtk_container_add(GTK_CONTAINER(tmp_3),tmp_2);
	gtk_signal_connect(GTK_OBJECT(tmp_2), "clicked", GTK_SIGNAL_FUNC(dialog_bt), "1");
    }
    dialog_exit=0;
    gtk_widget_show_all(tmp_1);
    dialog_code = 0;    
    for(;!dialog_exit;) rfsh_gtk();
        
    if(dialog_code) gtk_widget_destroy(tmp_1);

    return dialog_code;
}

void set_default(){
    SET_DIP_SWITCH(0);
    SET_PCB_WILLEM(0);
    SET_PORT_ADDR(LP0_ADDR);
    SET_C_IMAGE(NULL);


    SET_BT(CH_BT_PCB,GET_PCB_WILLEM);
    set_rd_bt_lp();
    draw_dip_switch();
    draw_pict();
    stat_rfsh();
}

void gui_run()
{
    device_menu_create(GET_WG(MB_P_DEVICE));
    gtk_widget_show_all(GET_WG(MAIN_WINDOW));
    SET_STARTAP(0);
    gtk_main();
}

void kill_me()
{
    printf("Papapa.\n");
    gtk_main_quit();
    free(GET_BUFFER);
    rem_all_modl(&mdl);
}

void dont_kill_me(GtkWidget *k, gpointer gd)
{
    gtk_widget_destroy(GTK_WIDGET(gd));
}

char gui_cmp_pls(int a,int b)
{
    char test = a < b;
    if(!test ) progress_bar_free();
    return test;
}
