/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "modules.h"
#include "27xx.xpms"

#define SIZE_2716	2048
#define SIZE_2732	4096
#define SIZE_2764	8192

void fin_message(char error, char *title, char *ok, char *err)
{
    dialog_box(title, error ? err : ok, "OK", NULL);
}

unsigned char read_eprom_byte(int addr, int t, char ce, char oe)
{
    unsigned char data;
    
    set_oe_pin(ce); set_ce_pin(oe);
    set_par_addr_pin(addr);
    us_delay(t);
    set_oe_pin(oe); set_ce_pin(ce);
    data = get_par_data_pin();
    us_delay(t);
    return data;
}

void read_27xx(char *pname, int size, int t1, int t2, char ce, char oe)
{
    int addr;
    
    vcc_off(); vpp_off();
    us_delay(t1);
    progress_bar_init(pname, size);
    vcc_on();
    set_par_addr_pin(0); set_par_data_pin(0);
    us_delay(t1);
    for(addr = 0; addr < size; addr++){
	progress_bar_set(addr);
	write_buffer(addr, read_eprom_byte(addr, t2, ce, oe));
    }
    progress_bar_free();
    vcc_off(); vpp_off();
    checksum();
    stat_rfsh();
}


void verify_27xx(char *pname, int size, int t1, int t2, char ce, char oe)
{
    int addr;
    char error;
    int a,b;
    
    vcc_off();
    vpp_off();
    us_delay(t1);
    progress_bar_init(pname,size);
    vcc_on();
    set_par_addr_pin(0);
    set_par_data_pin(0);
    us_delay(t1);
    error=0;
    for(addr = 0; addr < size; addr++){
	progress_bar_set(addr);
	if( (a = read_eprom_byte(addr, t2,ce,oe)) != (b = read_buffer(addr) & 0xff) ){
//printf("%i != %i, adres: %x\n",a,b,addr);
	    error = 1;
	    dialog_box("Weryfikacja","Uklad niezgodny z buforem","OK",NULL);
//	    set_status("Uklad nie jest skasowany");
	    break;    
	}
	us_delay(t2);
    }
    if(!error){
	    dialog_box("Weryfikacja","Uklad zgodny z buforem","OK",NULL);	    
//	    set_status("Uklad jest skasowany");	    
    }
    progress_bar_free();
    vcc_off();
    vpp_off();
    checksum();
    stat_rfsh();

}

void test_27xx(char *pname, int size, int t1, int t2, char ce, char oe)
{
    int addr;
    char error;

    vcc_off();
    vpp_off();
    us_delay(t1);
    progress_bar_init(pname,size);
    vcc_on();
    set_par_addr_pin(0);
    set_par_data_pin(0);
    us_delay(t1);
    error=0;
    for(addr = 0; addr < size; addr++){
	progress_bar_set(addr);
	if( read_eprom_byte(addr, t2,ce,oe) != 0xff ){
	    error = 1;
	    dialog_box("Test","Uklad nie jest w pelni skasowany","OK",NULL);
//	    set_status("Uklad nie jest skasowany");
	    break;    
	}
	us_delay(t2);
    }
    if(!error){
	    dialog_box("Test","Uklad jest czysty","OK",NULL);	    
//	    set_status("Uklad jest skasowany");	    
    }
    progress_bar_free();
    vcc_off();
    vpp_off();
    checksum();
    stat_rfsh();
}

void write_eprom_byte_2716(int addr, unsigned char data, unsigned int ts, unsigned int time)
{
    set_oe_pin(1); set_ce_pin(0);
    set_par_addr_pin(addr);
    set_par_data_pin(data);
    us_delay(ts);
    set_ce_pin(1);		
    us_delay(time);	/* 50ms */
    set_ce_pin(0);
}

void prog_2716()
{
    int addr;
    int a,b;
    
    vcc_off(); vpp_off();
    us_delay(1000);
    progress_bar_init("Programowanie EPROM 2716", SIZE_2716);
    vcc_on(); vpp_on();
    set_par_addr_pin(0);
    set_par_data_pin(0);
    us_delay(1000);
    for(addr = 0; addr < SIZE_2716; addr++){
	progress_bar_set(addr);
	if((read_buffer(addr) & 0xff) != 0xff )
	    write_eprom_byte_2716(addr, read_buffer(addr), 20, 50 * 1000);
	if( (a = read_eprom_byte(addr, 20,0,0)) != (b = read_buffer(addr) & 0xff) ){
//printf("%i != %i, adres: %x\n",a,b,addr);
	    dialog_box("Blad programowania !!!!","Blad zapisu","OK",NULL);
//	    set_status("Uklad nie jest skasowany");
	    break;    
	}
    }
    progress_bar_free();
    vcc_off();
    vpp_off();
    checksum();
    stat_rfsh();
}

void read_2716()
{
    read_27xx("Odczyt ukladu EPROM 2716",SIZE_2716,1000,10,0,0);
}

void verify_2716()
{
    verify_27xx("Weryfikacja ukladu EPROM 2716",SIZE_2716,1000,10,0,0);
}

void test_2716()
{
    test_27xx("Test czystoci ukladu EPROM 2716",SIZE_2716,1000,10,0,0);
}

void write_eprom_byte_oevpp(int addr, unsigned char data, unsigned int ts, unsigned int time)
{
    set_ce_pin(0);vpp_off();
    set_par_addr_pin(addr);
    set_par_data_pin(data);
    us_delay(ts);
    vpp_on();
    us_delay(time);	/* 50ms */
    set_ce_pin(0);
    vpp_off();
}


/********************************************************************************************/
void read_2732()
{
    read_27xx("Odczyt ukladu EPROM 2732",SIZE_2732,1000,1,0,0);
}

void verify_2732()
{
    verify_27xx("Weryfikacja ukladu EPROM 2732",SIZE_2732,1000,1,0,0);
}

void test_2732()
{
    test_27xx("Test czystoci ukladu EPROM 2732",SIZE_2732,1000,1,0,0);
}

void prog_2732()
{
    int addr;
    int a,b;
    
    vcc_off(); vpp_off();
    us_delay(1000);
    progress_bar_init("Programowanie EPROM 2732", SIZE_2732);
    vcc_on();
    set_par_addr_pin(0);
    set_par_data_pin(0);
    us_delay(1000);
    for(addr = 0; addr < SIZE_2732; addr++){
	progress_bar_set(addr);
	if((read_buffer(addr) & 0xff) != 0xff )
	    write_eprom_byte_oevpp(addr, read_buffer(addr), 20, 50 * 1000);
	if( (a = read_eprom_byte(addr, 20,0,0)) != (b = read_buffer(addr) & 0xff) ){
//printf("%i != %i, adres: %x\n",a,b,addr);
	    dialog_box("Blad programowania !!!!","Blad zapisu","OK",NULL);
//	    set_status("Uklad nie jest skasowany");
	    break;    
	}
    }
    progress_bar_free();
    vcc_off();
    vpp_off();
    checksum();
    stat_rfsh();

}


/********************************************************************************************/
void pgm_imp(char type,int tp, int ts)
{
    us_delay(ts);
    if(type==0)
	{ set_ce_pin(0); us_delay(tp); set_ce_pin(1); }
}

void prog_init(char type, int ts)
{
    set_par_addr_pin(0); set_par_data_pin(0);
    if(type == 0)
	{ set_ce_pin(1); set_oe_pin(1); }

    vcc_on(); vpp_on();
}

char byte_verify(unsigned int addr, char type, char ce, char oe)
{
    return (read_buffer(addr) & 0xff) != (read_eprom_byte(addr, 20,ce,oe) & 0xff);
}

void finish(){ vcc_off(); vpp_off(); checksum(); stat_rfsh(); }

void set_prog_data(unsigned int addr)
{
    set_par_addr_pin(addr);
    set_par_data_pin(read_buffer(addr));
}

void fast_prog(int size, char type, int tp, int ts, char ce, char oe)
{
    char X, error;
    unsigned int addr;    
    
    vcc_off(); vpp_off();
    prog_init(type,ts);

    progress_bar_init("Programowanie algorytmem FAST", size);
    addr  = 0;
    error = 0;
    for(addr=0; addr < size; addr++){
printf("ADDR: %x\n",addr);
	progress_bar_set(addr);
	set_prog_data(addr);
	us_delay(ts);	
        pgm_imp(type,tp,ts);
	X = 0;
	while(byte_verify(addr, type, ce, oe)){
	    set_prog_data(addr);
	    X++;
	    if(X == 25 ) {error=1; break;}
	    pgm_imp(type, 3 * X * tp, ts);	
	}
	if(X == 25) break;
    }
    progress_bar_free();
    if(!error){
	progress_bar_init("Weryfikacja zapisu", size);
	for(addr = 0; addr < size; addr++ ){
		progress_bar_set(addr);
		if( byte_verify(addr, type, ce, oe) ){ error = 1; break; }
	}
	progress_bar_free();
    }
    fin_message(error, "Programowanie zakonczone",
		       "Poprawnie zaprogramowane", "Blad programowania !!!!");
    finish();    
}

void normal_prog(int size, char type, int tp, int ts, char ce, char oe)
{
    char X, error;
    unsigned int addr;    
    
    vcc_off(); vpp_off();
    prog_init(type,ts);

    progress_bar_init("Programowanie algorytmem NORMAL", size);
    addr  = 0;
    error = 0;

    do{
	progress_bar_set(addr);

	set_par_addr_pin(addr);
	set_par_data_pin(read_buffer(addr));
	us_delay(ts);	

        pgm_imp(type,tp,ts);
	if(byte_verify(addr, type, ce, oe)) {error=1; break;}
    }while( ++addr != size);

    progress_bar_free();

    progress_bar_init("Weryfikacja zapisu", size);
    for(addr = 0; addr < size; addr++ )	if( byte_verify(addr, type, ce, oe) ){ error = 1; break; }
    progress_bar_free();

    fin_message(error, "Programowanie zakonczone",
		       "Poprawnie zaprogramowane", "Blad programowania !!!!");
    finish();    
}

/********************************************************************************************/
void read_2764(){ read_27xx("Odczyt ukladu EPROM 2764",SIZE_2764,1000,1,1,0); }
void verify_2764(){ verify_27xx("Weryfikacja ukladu EPROM 2764",SIZE_2764,1000,1,1,0); }
void test_2764(){ test_27xx("Test czystoci ukladu EPROM 2764",SIZE_2764,1000,1,1,0); }
void prog_2764(){ 
//    normal_prog(SIZE_2764,0,50*1000,10,1,0);
    fast_prog(SIZE_2764,0,1000,1,1,0); 
}

/********************************************************************************************/
void prog_27xx(){}


void init_module(void)
{
    int img;
    INIT_MODULE("Init 27xx module\n");

    img = register_image(d_2716_xpm);
    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/24 pin","2716", 0, SIZE_2716);
    INIT_IMAGE_SET_IDX(img,img);
    D_FUNC_INIT_SET(read_2716, prog_2716, NULL, verify_2716);
    D_TEST(test_2716);
    SET_DIPSW(0x50);
    D_REGISTER;

    img = register_image(d_2732_xpm);
    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/24 pin","2732", 0, SIZE_2732);
    INIT_IMAGE_SET_IDX(img,img);
    D_FUNC_INIT_SET(read_2732, prog_27xx, NULL, verify_2732);
    D_TEST(test_2732);
    SET_DIPSW(0x30);
    D_REGISTER;

    img = register_image(d_2764_xpm);
    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/28 pin","2764", 0, 8192);
    INIT_IMAGE_SET_IDX(img,img);
    D_FUNC_INIT_SET(read_2764, prog_2764, NULL, verify_2764);
    D_TEST(test_2764);
    SET_DIPSW(0x948);
    D_REGISTER;

/*
    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/28 pin","27128", 0, 16384);
    INIT_IMAGE_SET_IDX(D_2764_WILLEM_IMG,D_2764_PCB3_IMG);
    D_FUNC_INIT_SET(read_27xx, prog_27xx, NULL, verify_27xx);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/28 pin","27256", 0, 32768);
    INIT_IMAGE_SET_IDX(D_2764_WILLEM_IMG,D_2764_PCB3_IMG);
    D_FUNC_INIT_SET(read_27xx, prog_27xx, NULL, verify_27xx);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/28 pin","27512", 0, 65536);
    INIT_IMAGE_SET_IDX(D_2764_WILLEM_IMG,D_2764_PCB3_IMG);
    D_FUNC_INIT_SET(read_27xx, prog_27xx, NULL, verify_27xx);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/32 pin","27C010", 0, 131072);
    INIT_IMAGE_SET_IDX(D_27010_WILLEM_IMG,D_27010_PCB3_IMG);
    D_FUNC_INIT_SET(read_27xx, prog_27xx, NULL, verify_27xx);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/32 pin","27C020", 0, 262144);
    INIT_IMAGE_SET_IDX(D_27010_WILLEM_IMG,D_27010_PCB3_IMG);
    D_FUNC_INIT_SET(read_27xx, prog_27xx, NULL, verify_27xx);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/EPROM/32 pin","27C040", 0, 524288);
    INIT_IMAGE_SET_IDX(D_27010_WILLEM_IMG,D_27010_PCB3_IMG);
    D_FUNC_INIT_SET(read_27xx, prog_27xx, NULL, verify_27xx);
    D_REGISTER;
*/
}
