/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "parport.h"
#include "willem.h"
#include "timer.h"

/* programmer control */
void vcc_on()
{
    set_bit(PC,PP_16);
}

void vcc_off()
{
    clr_bit(PC,PP_16);
}

void vpp_on()
{
    set_bit(PC,PP_01);
}

void vpp_off()
{
    clr_bit(PC,PP_01);
}

void set_dip_sw(int dip){ DEBUG("[DM]Set Dip SW: 0x%x\n",dip);} /* It's a dummy function (maybe for future version ?) */
void set_vpp_voltage(float voltage){ DEBUG("[DM]Set VPP value: %f\n",voltage);} /* It's a dummy function (maybe for future version ?) */


/* ZIF-32 - test settings */
void set_par_data_pin(char data)
{
    clr_bit(PC,PP_14);
    set_par_port(PA,data);
}

void set_par_addr_pin(int addr)
{
    int mask;

    set_bit(PC,PP_14);
    mask = 0x800000;    
    clr_bit(PA, PP_03 + PP_02);
    while(mask){
	us_delay(TA_01);
	if(addr & mask) set_bit(PA,PP_03); else clr_bit(PA,PP_03);
	us_delay(TA_02);
	set_bit(PA,PP_02);
	us_delay(TA_03);
	clr_bit(PA,PP_02);
	mask = mask >> 1;
    }
    clr_bit(PC,PP_14);
}

void set_we_pin(char bool)
{
    if(bool) set_bit(PC,PP_17); else clr_bit(PC,PP_17);
}

void set_oe_pin(char bool)
{
    if(bool) clr_bit(PC,PP_14); else set_bit(PC,PP_14);
}

void set_ce_pin(char bool)
{
    if(bool) set_bit(PC,PP_17); else clr_bit(PC,PP_17);
}

char get_par_data_pin(void)
{
    unsigned char data,i;
    
    data = 0;
    set_bit(PC,PP_14);
    set_bit(PA,PP_04);
    clr_bit(PA,PP_03);    
    us_delay(TD_01);
    set_bit(PA,PP_03);    
    us_delay(TD_02);
    clr_bit(PA,PP_04);        
    us_delay(TD_03);
    set_bit(PA,PP_04);        
    clr_bit(PA,PP_03);        

    for(i = 0; i < 8; i++){
	us_delay(TD_04);
	if(get_bit(PB, PP_10))
		data |= (0x80 >> i); 
	clr_bit(PA,PP_04);
	us_delay(TD_05);
	set_bit(PA,PP_04);
    }

//    DEBUG("[DM]Get ZIF32 DATA: 0x%x\n",data);

    return data;
}

/* serial 93Cxx */
void set_cs_pin(char bool)
{
    if(bool) set_bit(PC,PP_17); else clr_bit(PC,PP_17);    
}

void set_clk_pin(char bool)
{
    clr_bit(PC,PP_14);
    if(bool) set_bit(PA,PP_03); else clr_bit(PA,PP_03);
}

void set_di_pin(char bool)
{
    if(bool) clr_bit(PA,PP_01); else set_bit(PA,PP_01);
}

char get_do_pin(void)
{
    char d_out = 0;

    d_out = get_bit(PB,PP_11) ? 1:0;
    return d_out;
}
